﻿using NetMQ;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace NetMQAndProtocolBuffer.PublisherPattern.Client
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            const int zipToSubscribeTo = 10001;
            const int iterations = 10;

            int curIndex = 1;
            int totalTemp = 0;
            int totalHumidity = 0;

            WriteLine(string.Format("Collecting updates for weather service for zipcode {0}...", zipToSubscribeTo));

            var sendTask = Task.Factory.StartNew(() =>
          {
              using (var context = NetMQContext.Create())
              using (var subscriber = context.CreateSubscriberSocket())
              {
                  subscriber.Connect("tcp://127.0.0.1:5556");
                  subscriber.SubscribeToAnyTopic(); // No Command Filter, warn if not set thie method SubscribeToAnyTopic, it will receive nothing.

                  while (true)
                  {
                      if (curIndex > iterations) break;

                      var weatherMsg = new NetMQAndProtocolBuffer.PBProtocol.WeatherMessage();
                      var receivedBytes = subscriber.Receive();
                      using (var sm = new MemoryStream(receivedBytes))
                      {
                          weatherMsg = ProtoBuf.Serializer.Deserialize<NetMQAndProtocolBuffer.PBProtocol.WeatherMessage>(sm);
                      }

                      // "zip temp relh" ... "10001 84 23" -> ["10001", "84", "23"]
                      string[] split = weatherMsg.Content.Split(new[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                      int cmdId = int.Parse(split[0]);
                      if (weatherMsg.Command == PBProtocol.WeatherMessage.CommandType.Weather)
                      {
                          if (cmdId == zipToSubscribeTo)
                          {
                              curIndex++;
                              WriteLine(string.Format("Subscriber receive message: {0}", weatherMsg.Content));
                              totalTemp += int.Parse(split[1]);
                              totalHumidity += int.Parse(split[2]);
                          }
                      }
                  }
              }

              WriteLine("Total: ");
              WriteLine(string.Format("Average temperature was: {0}", totalTemp / iterations));
              WriteLine(string.Format("Average relative humidity was: {0}", totalHumidity / iterations));
          });
        }

        /// <summary>
        /// Writes the line.
        /// </summary>
        /// <param name="msg">The MSG.</param>
        private void WriteLine(string msg)
        {
            this.Dispatcher.Invoke(new Action(() => { this.Txt_Output.Text += msg + System.Environment.NewLine; }));
        }

        /// <summary>
        /// Handles the TextChanged event of the Txt_Output control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="TextChangedEventArgs"/> instance containing the event data.</param>
        private void Txt_Output_TextChanged(object sender, TextChangedEventArgs e)
        {
            this.Txt_Output.ScrollToEnd();
        }

    }
}
