﻿using System;
using System.Globalization;
using NetMQ;

namespace WeatherUpdateClient
{
    internal static class WeatherUpdateClient
    {
        private static void Main()
        {
            const int zipToSubscribeTo = 10001;
            const int iterations = 100;

            int totalTemp = 0;
            int totalHumidity = 0;

            Console.WriteLine("Collecting updates for weather service for zipcode {0}...", zipToSubscribeTo);

            using (var context = NetMQContext.Create())// 创建全局NetMQ句柄，建议唯一，使用完毕及时回收。 
            using (var subscriber = context.CreateSubscriberSocket())// 创建Publisher-Subscriber模式的客户端监听。
            {
                subscriber.Connect("tcp://127.0.0.1:5556");// 连接到指定Socket
                subscriber.Subscribe(zipToSubscribeTo.ToString(CultureInfo.InvariantCulture));// 这里创建消息内容的过滤，如果不包含“zipToSubscribeTo”值则不接收消息。

                for (int i = 0; i < iterations; i++)
                {
                    string results = subscriber.ReceiveString(); // 如果消息以“zipToSubscribeTo”开头，则会返回整条信息。
                    Console.Write(".");

                    // "zip temp relh" ... "10001 84 23" -> ["10001", "84", "23"]
                    string[] split = results.Split(new[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);// 按照固定模式解码。

                    int zip = int.Parse(split[0]);
                    if (zip != zipToSubscribeTo)
                    {
                        throw new Exception(string.Format("Received message for unexpected zipcode: {0} (expected {1})", zip, zipToSubscribeTo));
                    }

                    totalTemp += int.Parse(split[1]);
                    totalHumidity += int.Parse(split[2]);
                }
            }

            Console.WriteLine();
            Console.WriteLine("Average temperature was: {0}", totalTemp / iterations);
            Console.WriteLine("Average relative humidity was: {0}", totalHumidity / iterations);
            Console.Read();
        }
    }
}